@@ -1,4 +1,4 @@
-
+import pysam
 
 class BAM():
     def __init__(self, filename, title=''):
@@ -7,6 +7,7 @@ class BAM():
             self.title = title
         else:
             self.title = self.filename.split('/')[-1]
+        self.samAlign = pysam.AlignmentFile(self.filename, "rb")
 
     def __str__(self):
         return self.title
